/*
 *  video_out_dc.c
 *
 *  Copyright (C) 2001 Florian 'Proff' Schulze <florian.proff.schulze@gmx.net>
 *
 *  A mpeg2dec display driver that does output through the
 *  Dreamcast Tile Accelerator
 *
 *  This file is NOT! part of oms, free DVD and Video player.
 *  It is derived from the mpeg2dec SDL video output plugin.
 *  Adopted for mpeg2dec from Dominik Schnitzer, Jan. 24. 2001
 *	
 *  oms is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *   
 *  this oms output plugin is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 
 *
 *  Changes:
 *    Florian - January 28, 2001.
 *    - first version
 * 
 */

#include "config.h"

#ifdef LIBVO_DC

#define TEX_SIZE 512

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <kallisti/libk.h>
#include <kallisti/abi/ta.h>

#include "video_out.h"
#include "video_out_internal.h"

/** Private SDL Data structure **/

abi_ta_t	*ta = NULL;

typedef struct dc_frame_s {
    vo_frame_t vo;
} dc_frame_t;

typedef struct dc_instance_s {
    vo_instance_t vo;
    int prediction_index;
    dc_frame_t frame[3];

	unsigned int video_tex;
    uint8_t *tex_mem;
    uint32_t width;
    uint32_t height;
	
	unsigned texture_pool;

} dc_instance_t;

static dc_instance_t dc_static_instance;	/* FIXME */

extern vo_instance_t dc_vo_instance;

/** libvo Plugin functions **/


unsigned int texture_alloc(uint32 size)
{
    unsigned int rv;
    
    rv = dc_static_instance.texture_pool;
    
    /* Align to 8 bytes */  
    size = (size + 7) & (~7);
    dc_static_instance.texture_pool += size;
    
    printf("Allocating texture at %08x\r\n", rv);
    return rv;
}

/**
 * Draw a frame to the SDL YUV overlay and checks for events.
 *
 *   params : frame == vo_frame_t to draw
 *  returns : doesn't return
 **/

static void dc_draw_frame (vo_frame_t * _frame)
{
    poly_hdr_t poly;
    dc_frame_t * frame;
    dc_instance_t * this;
    vertex_ot_t vert;
    int x,y;
    static uint32_t test;

    frame = (dc_frame_t *)_frame;
    this = (dc_instance_t *)frame->vo.this;
	
	for (y=0; y<this->height; y++)
	{
	    uint32_t *tex;
	    uint8_t *cy;
    	uint8_t *cu;
    	uint8_t *cv;
    	
		cy=&frame->vo.base[0][y*this->width];
		cu=&frame->vo.base[1][y*this->width/4];
		cv=&frame->vo.base[2][y*this->width/4];
		tex=(uint32_t *)&this->tex_mem[y*TEX_SIZE*2];
		for (x=0; x<(this->width/2); x++)
			*tex++ = 
				(uint32_t)(((uint8_t)*cu++)) |
				(uint32_t)(((uint8_t)*cy++)<<8) |
				(uint32_t)(((uint8_t)*cv++)<<16) |
				(uint32_t)(((uint8_t)*cy++)<<24);
		cu=&frame->vo.base[1][y*this->width/4];
		cv=&frame->vo.base[2][y*this->width/4];
		y++;
		tex=(uint32_t *)&this->tex_mem[y*TEX_SIZE*2];
		for (x=0; x<(this->width/2); x++)
			*tex++ = 
				(uint32_t)(((uint8_t)*cu++)) |
				(uint32_t)(((uint8_t)*cy++)<<8) |
				(uint32_t)(((uint8_t)*cv++)<<16) |
				(uint32_t)(((uint8_t)*cy++)<<24);
	}
	
    /* Begin opaque polygons */
    ta->begin_render();
    
    /* Send polygon header to the TA using store queues */
    ta->poly_hdr_txr(&poly, TA_TRANSLUCENT, TA_YUV422, TEX_SIZE, TEX_SIZE, this->video_tex, TA_BILINEAR_FILTER);
    ta->commit_poly_hdr(&poly);

    /* Draw frame */
    
    vert.r = vert.g = vert.b = vert.a = 1.0f;
    vert.oa = vert.or = vert.ob = 0.0f;
    vert.flags = TA_VERTEX_NORMAL;

    vert.x = 0.0f;
    vert.y = 479.0f;
    vert.z = 2.0f;
    vert.u = 0.0;
    vert.v = (float)this->height/(float)TEX_SIZE;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 0.0f;
    vert.y = 0.0f;
    vert.z = 2.0f;
    vert.u = 0.0;
    vert.v = 0.0;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 639.0f;
    vert.y = 479.0f;
    vert.z = 2.0f;
    vert.u = (float)this->width/(float)TEX_SIZE;
    vert.v = (float)this->height/(float)TEX_SIZE;
    ta->commit_vertex(&vert, sizeof(vert));
    
    vert.x = 639.0f;
    vert.y = 0.0f;
    vert.z = 2.0f;
    vert.u = (float)this->width/(float)TEX_SIZE;
    vert.v = 0.0;
    vert.flags = TA_VERTEX_EOL;
    ta->commit_vertex(&vert, sizeof(vert));
    
    /* Begin translucent polygons */
    ta->commit_eol();
    
    /* Finish up */
    ta->commit_eol();
    ta->finish_frame();
#if 0
    sdl_frame_t * frame;
    sdl_instance_t * this;

    frame = (sdl_frame_t *)_frame;
    this = (sdl_instance_t *)frame->vo.this;
	
    SDL_LockYUVOverlay (overlay);

	memcpy(&overlay->pixels[0][0],&frame->vo.base[0][0],overlay->w*overlay->h*2);

    /* Unlock the frame - the frame is 100% filled with data to display 
     * We Lock it again when the frame was displayed. */
    SDL_UnlockYUVOverlay (overlay);

    /* blit to the YUV overlay */
    SDL_DisplayYUVOverlay (overlay, &(this->surface->clip_rect));
	
    /* check for events */
    //check_events ();
#endif    
}

static int dc_alloc_frames (dc_instance_t * this, int width, int height)
{
    int i;
	uint8_t *p;

    for (i = 0; i < 3; i++)
    {
		p=malloc(width*height*2);
		this->frame[i].vo.base[0] = &p[0];
		this->frame[i].vo.base[2] = &p[width*height];
		this->frame[i].vo.base[1] = &p[width*height+(width*height/4)];
		this->frame[i].vo.copy = NULL;
		this->frame[i].vo.draw = dc_draw_frame;
		this->frame[i].vo.this = (vo_instance_t *)this;
    }
	
    return 0;
}


static void dc_free_frames (dc_instance_t * this)
{
}

vo_frame_t * dc_get_frame (vo_instance_t * _this, int prediction)
{
    dc_instance_t * this;

    this = (dc_instance_t *)_this;

    if (!prediction)
	return (vo_frame_t *)(this->frame + 2);
    else {
	this->prediction_index ^= 1;
	return (vo_frame_t *)(this->frame + this->prediction_index);
    }
}

/**
 * Open and prepare SDL output for dedicated Software and aalib out
 *
 *    params : this == instance to initialize
 *             width, height == width and height of output window
 *   returns : NULL on failure, a valid vo_instance_t on success
 **/

vo_instance_t * vo_dc_setup (vo_instance_t * _this, int width, int height)
{
    dc_instance_t * this;
    
    if (_this != NULL)
		return NULL;
		
	printf("Opening TA lib\r\n");
	if (ta == NULL) ta = lib_open("ta");
    if (ta == NULL) { printf("Can't open TA lib\r\n"); return NULL; }
    
    this = &dc_static_instance;

    this->texture_pool = 0;

	printf("Allocating texture\r\n");
    this->video_tex = texture_alloc(TEX_SIZE*TEX_SIZE*2);
    this->tex_mem = ta->texture_map(this->video_tex);
	this->width = width;
	this->height = height;
	    
	printf("Allocating frames\r\n");
    if (dc_alloc_frames (this, width, height))
    {
		fprintf (stderr, "dc could not allocate frame buffers\n");
		return NULL;
    }
	 
    this->vo = dc_vo_instance;

    return (vo_instance_t *)this;
}

/**
 * Close SDL, Cleanups, Free Memory.
 *
 *    params : this == sdl vo instance to close
 *   returns : non-zero on error, zero on success
 **/

static void dc_close (vo_instance_t * _this)
{
#if 0
    sdl_instance_t * this = (sdl_instance_t *)_this;

    sdl_free_frames (this);

	SDL_FreeYUVOverlay (overlay);

    /* Free our blitting surface */
    if (this->surface)
	SDL_FreeSurface(this->surface);
	
    /* Cleanup SDL */
    SDL_QuitSubSystem(SDL_INIT_VIDEO);
#endif    
}




vo_instance_t dc_vo_instance = {
    vo_dc_setup, dc_close, dc_get_frame
};

#endif
