/*
 * mpeg2dec.c
 * Copyright (C) 1999-2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *
 * This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
 *
 * mpeg2dec is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * mpeg2dec is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include <stdio.h>
//#include <stdlib.h>
//#include <sys/time.h>
//#include <signal.h>
#include <string.h>
//#include <errno.h>
#include <inttypes.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
#include <unistd.h>
#endif
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/fs.h>

#include "video_out.h"
#include "mpeg2.h"

#define BUFFER_SIZE 262144
static uint8_t buffer[BUFFER_SIZE];
//static FILE * in_file;
static int input_file_handle;
static abi_fs_t *fslib;
abi_maple_t	*maple = NULL;
static uint32_t frame_counter = 0;
static int quit = 0;

//static struct timeval tv_beg, tv_end, tv_start;
static uint32_t tv_beg, tv_end, tv_start;
static uint32_t elapsed;
static uint32_t total_elapsed;
static uint32_t last_count = 0;
static uint32_t demux_dvd = 0;
static mpeg2dec_t mpeg2dec;
static vo_setup_t * output_setup = NULL;

void exit(int i)
{
	quit=1;
}

static void print_fps (int final) 
{
    int fps, tfps, frames;
	
    //gettimeofday (&tv_end, NULL);
	tv_end = timer_count(TMU1);

    if (frame_counter++ == 0) {
	tv_start = tv_beg = tv_end;
    }

/*
    elapsed = (tv_end.tv_sec - tv_beg.tv_sec) * 100 +
	(tv_end.tv_usec - tv_beg.tv_usec) / 10000;
    total_elapsed = (tv_end.tv_sec - tv_start.tv_sec) * 100 +
	(tv_end.tv_usec - tv_start.tv_usec) / 10000;
*/
	elapsed = (tv_end - tv_beg) / 10;
	total_elapsed = (tv_end - tv_start) / 10;

    if (final) {
	if (total_elapsed) 
	    tfps = frame_counter * 10000 / total_elapsed;
	else
	    tfps = 0;

	fprintf (stderr,"\n%d frames decoded in %d.%02d "
		 "seconds (%d.%02d fps)\n", frame_counter,
		 total_elapsed / 100, total_elapsed % 100,
		 tfps / 100, tfps % 100);

	return;
    }

    if (elapsed < 500)	/* only display every 0.50 seconds */
	return;

    tv_beg = tv_end;
    frames = frame_counter - last_count;

    fps = frames * 10000 / elapsed;			/* 100x */
    tfps = frame_counter * 10000 / total_elapsed;	/* 100x */

    fprintf (stderr, "%d frames in %d.%02d sec (%d.%02d fps), "
	     "%d last %d.%02d sec (%d.%02d fps)\033[K\r", frame_counter,
	     total_elapsed / 100, total_elapsed % 100,
	     tfps / 100, tfps % 100, frames, elapsed / 100, elapsed % 100,
	     fps / 100, fps % 100);
    last_count = frame_counter;
}

/* 
static void signal_handler (int sig)
{
    print_fps (1);
    signal (sig, SIG_DFL);
    raise (sig);
}
*/

static void print_usage (char * argv[])
{
    int i;

    fprintf (stderr, "usage: %s [-o mode] [-s] file\n"
	     "\t-s\tsystem stream (.vob file)\n"
	     "\t-o\tvideo_output mode\n", argv[0]);

    for (i = 0; video_out_drivers[i].name; i++)
	fprintf (stderr, "\t\t\t%s\n", video_out_drivers[i].name);

    exit (1);
}

static void handle_args (int argc, char * argv[])
{
#if 0
    int c;
    int i;

    while ((c = getopt (argc,argv,"so:")) != -1) {
	switch (c) {
	case 'o':
	    for (i=0; video_out_drivers[i].name != NULL; i++) {
		if (strcmp (video_out_drivers[i].name, optarg) == 0)
		    output_setup = video_out_drivers[i].setup;
	    }
	    if (output_setup == NULL) {
		fprintf (stderr, "Invalid video driver: %s\n", optarg);
		print_usage (argv);
	    }
	    break;

	case 's':
	    demux_dvd = 1;
	    break;

	default:
	    print_usage (argv);
	}
    }

    /* -o not specified, use a default driver */
    if (output_setup == NULL)
	output_setup = video_out_drivers[0].setup;

    if (optind < argc) {
	in_file = fopen (argv[optind], "rb");
	if (!in_file) {
	    fprintf (stderr, "%s - couldnt open file %s\n", strerror (errno),
		     argv[optind]);
	    exit (1);
	}
    } else
	in_file = stdin;

    mpeg2_init (&mpeg2dec, output_setup);
#endif
    output_setup = video_out_drivers[0].setup;
//#define filename "/cd/mpeg/eternala.mpg"
//#define filename "/cd/mpeg/engshenm.mpg"
#define filename "/cd/diamond1.mpg"
	input_file_handle = fslib->open(filename, O_RDONLY);
	printf("Hnd is %08x\r\n", input_file_handle);
	if (input_file_handle == 0) {
		printf("Can't open file "filename" for reading.\r\n");
		return;
	}
#undef filename	
	demux_dvd=1;
    mpeg2_init (&mpeg2dec, output_setup);
}

static void ps_loop (void)
{
	cont_cond_t cond;
	
    static int mpeg1_skip_table[16] = {
	     1, 0xffff,      5,     10, 0xffff, 0xffff, 0xffff, 0xffff,
	0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff
    };

    uint8_t * buf;
    uint8_t * end;
    uint8_t * tmp1;
    uint8_t * tmp2;
    int complain_loudly;

    complain_loudly = 1;
    buf = buffer;

    do {
		if (maple->cont_get_cond(maple->first_controller(), &cond) < 0)
	    	break;
	
		if (!(cond.buttons & CONT_START))
	    	break;

		if (quit)
			break;
				
	//end = buf + fread (buf, 1, buffer + BUFFER_SIZE - buf, in_file);
	end = buf + fslib->read (input_file_handle, buf, buffer + BUFFER_SIZE - buf);
	buf = buffer;

	while (buf + 4 <= end) {
	    /* check start code */
	    if (buf[0] || buf[1] || (buf[2] != 0x01)) {
		if (complain_loudly) {
		    fprintf (stderr, "missing start code at %#lx\n",
			     fslib->tell (input_file_handle) - (end - buf));
//			     ftell (in_file) - (end - buf));
		    if ((buf[0] == 0) && (buf[1] == 0) && (buf[2] == 0))
			fprintf (stderr, "this stream appears to use "
				 "zero-byte padding before start codes,\n"
				 "which is not correct according to the "
				 "mpeg system standard.\n"
				 "mp1e was one encoder known to do this "
				 "before version 1.8.0.\n");
		    complain_loudly = 0;
		}
		buf++;
		continue;
	    }

	    switch (buf[3]) {
	    case 0xb9:	/* program end code */
		return;
	    case 0xba:	/* pack header */
		/* skip */
		if ((buf[4] & 0xc0) == 0x40)	/* mpeg2 */
		    tmp1 = buf + 14 + (buf[13] & 7);
		else if ((buf[4] & 0xf0) == 0x20)	/* mpeg1 */
		    tmp1 = buf + 12;
		else if (buf + 5 > end)
		    goto copy;
		else {
		    fprintf (stderr, "weird pack header\n");
		    exit (1);
		}
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;
	    case 0xe0:	/* video */
		tmp2 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp2 > end)
		    goto copy;
		if ((buf[6] & 0xc0) == 0x80)	/* mpeg2 */
		    tmp1 = buf + 9 + buf[8];
		else {	/* mpeg1 */
		    for (tmp1 = buf + 6; *tmp1 == 0xff; tmp1++)
			if (tmp1 == buf + 6 + 16) {
			    fprintf (stderr, "too much stuffing\n");
			    buf = tmp2;
			    break;
			}
		    if ((*tmp1 & 0xc0) == 0x40)
			tmp1 += 2;
		    tmp1 += mpeg1_skip_table [*tmp1 >> 4];
		}
		if (tmp1 < tmp2) {
		    int num_frames;

		    num_frames = mpeg2_decode_data (&mpeg2dec, tmp1, tmp2);
		    while (num_frames--)
			print_fps (0);
		}
		buf = tmp2;
		break;
	    default:
		if (buf[3] < 0xb9) {
		    fprintf (stderr,
			     "looks like a video stream, not system stream\n");
		    exit (1);
		}
		/* skip */
		tmp1 = buf + 6 + (buf[4] << 8) + buf[5];
		if (tmp1 > end)
		    goto copy;
		buf = tmp1;
		break;
	    }
	}

	if (buf < end) {
	copy:
	    /* we only pass here for mpeg1 ps streams */
	    memmove (buffer, buf, end - buf);
	}
	buf = buffer + (end - buf);

    } while (end == buffer + BUFFER_SIZE);
}

static void es_loop (void)
{
#if 0
    uint8_t * end;
    int num_frames;
		
    do {
	end = buffer + fread (buffer, 1, BUFFER_SIZE, in_file);

	num_frames = mpeg2_decode_data (&mpeg2dec, buffer, end);

	while (num_frames--)
	    print_fps (0);

	} while (end == buffer + BUFFER_SIZE);
#endif
}

uint32_t mm_accel (void)
{
    return 0;
}

int main (int argc,char *argv[])
{
/*
    fprintf (stderr, PACKAGE"-"VERSION
	     " (C) 2000 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>\n");
*/
	maple = lib_open("maple");
	if (maple == NULL) { printf("Can't open Maple lib\r\n"); return -1; }

	fslib = lib_open("fs");
	printf("Initialized fs_abi.. get back %08x\r\n", fslib);

    timer_prime(TMU1,1000,0);
    timer_start(TMU1);
    
    handle_args (argc, argv);

    //signal (SIGINT, signal_handler);

    //gettimeofday (&tv_beg, NULL);
	tv_beg = timer_count(TMU1);

    if (demux_dvd)
		ps_loop ();
    else
		es_loop ();

    mpeg2_close (&mpeg2dec);
    print_fps (1);
  	fslib->close(input_file_handle);
  	lib_close(fslib);

  	input_file_handle = -1;
    timer_stop(TMU1);
    return 0;
}

#if 0
/* KallistiOS 0.7

   test1.c
   (c)2000 Dan Potter

*/

static char id[] = "KOS $Id: test1.c,v 1.1 2000/11/09 05:22:44 bard Exp $";

#include <stdio.h>
#include <kallisti/libk.h>
#include <kallisti/abi/fs.h>

abi_fs_t *fslib;

char buffer[2048];
int main(int argc, char **argv) {
	dirent_t	*de;
	uint32		hnd;
	int 		amt;
	int			i;

	printf("Hello there! Inside main...\r\n");

	fslib = lib_open("fs");
	printf("Initialized fs_abi.. get back %08x\r\n", fslib);

	printf("Opening /cd/\r\n");
	hnd = fslib->open("/cd/", O_RDONLY | O_DIR);
	printf("Hnd is %08x\r\n", hnd);
	while ( (de = fslib->readdir(hnd)) ) {
		printf("%s / %d bytes\r\n", de->name, de->size);
	}
	fslib->close(hnd);

	/* Read and print a file called README.TXT (if any) */
	hnd = fslib->open("/cd/doom/iwad/doom.wad", O_RDONLY);
	printf("Hnd is %08x\r\n", hnd);
	if (hnd == 0) {
		printf("Can't open file /cd/doom/iwad/doom.wad for reading.\r\n");
		return;
	}
	i=0;
	while ( (amt = fslib->read(hnd, buffer, 2048)) ) {
		i=i+amt;
		printf("%i\r\n", i);
	}
	fslib->close(hnd);

	lib_close(fslib);
	printf("Program exiting.\r\n");
}

#endif
